@@ -1,64 +1,57 @@
-#' @title Weighted average of SHAP values and weighted SHAP confidence intervals
-#'        for a grid of fine-tuned models or base-learners of a stacked ensemble
-#'        model
-#' @description Weighted average of SHAP values and weighted SHAP confidence intervals
-#'              provide a measure of feature importance for a grid of fine-tuned models
-#'              or base-learners of a stacked ensemble model. Instead of reporting
-#'              relative SHAP contributions for a single model, this function
-#'              takes the variability in feature importance of multiple models
-#'              into account and computes weighted mean and confidence intervals
-#'              for each feature, taking the performance metric of each model as
-#'              the weight. The function also provides a plot of the weighted
-#'              SHAP values and confidence intervals. Currently only models
-#'              trained by h2o machine learning software or autoEnsemble
-#'              package are supported.
-#' @param models H2O search grid, AutoML grid, or a character vector of H2O model IDs.
-#'               the \code{"h2o.get_ids"} function from \code{"h2otools"} can retrieve
-#'               the IDs from grids.
-#' @param newdata h2o frame (data.frame). the data.frame must be already uploaded
-#'                on h2o server (cloud). when specified, this dataset will be used
-#'                for evaluating the models. if not specified, model performance
-#'                on the training dataset will be reported.
-#' @param performance_metric character, specifying the performance metric to be
-#'                           used for weighting the SHAP values (mean and 95% CI). The default is
-#'                           "r2" (R Squared).
-#'                           For binary classification, other options include
-#'                           "aucpr" (area under the precision-recall curve),
-#'                           "auc" (area under the ROC curve),
-#'                           and "f2" (F2 score).
-#' @param standardize_performance_metric logical. if TRUE, performance_metric, which is
-#'                                       used as weights vector is standardized such
-#'                                       that the sum of the weights vector would be equal
-#'                                       to the length of the vector. the default value
-#'                                       is FALSE.
-#' @param performance_type character, specifying where the performance metric should
-#'                         be retrieved from. "train" means the performance of the
-#'                         training process should be reported, "valid" indicates that
-#'                         the performance of the validation process should be reported,
-#'                         and "xval" means the cross-validation performance to be
-#'                         retrieved.
-#' @param minimum_performance the minimum performance metric for a recognizable model.
-#'                            any model with performance equal or lower than this
-#'                            argument will have weight of zero in computing the
-#'                            weighted mean and CI SHAP values. the default value is
-#'                            zero.
-#' @param method character, specifying the method used for identifying the most
-#'               important features according to their weighted SHAP values.
-#'               The default selection method is "lowerCI", which includes
-#'               features whose lower weighted confidence interval exceeds the
-#'               predefined 'cutoff' value (default is 0).
-#'               Alternatively, the "mean" option can be specified, indicating
-#'               any feature with normalized weighted mean SHAP contribution above
-#'               the specified 'cutoff' should be selected. Another
-#'               alternative options is "shapratio", a method that filters
-#'               for features where the proportion of their relative weighted SHAP
-#'               value exceeds the 'cutoff'. This approach calculates the relative
-#'               contribution of each feature's weighted SHAP value against the
-#'               aggregate of all features, with those surpassing the 'cutoff'
-#'               being selected as top feature.
+#' @title Weighted Mean SHAP Ratio and Confidence Interval for a ML Grid
+#'        of Fine-Tuned Models or Base-Learners of a Stacked Ensemble Model
+#'
+#' @description
+#' Calculates weighted mean SHAP ratios and confidence intervals to assess feature importance
+#' across a collection of models (e.g., a grid of fine-tuned models or base-learners
+#' in a stacked ensemble). Rather than reporting relative SHAP contributions for
+#' only a single model, this function accounts for variability in feature importance
+#' across multiple models. Each model's performance metric is used as a weight.
+#' The function also provides a plot of weighted SHAP values with confidence intervals.
+#' Currently, only models trained by the \code{h2o} machine learning platform,
+#' \code{autoEnsemble}, and the \code{HMDA} R packages are supported.
+#'
+#' @details
+#'    The function works as follows:
+#'    \enumerate{
+#'      \item SHAP contributions are computed at the individual level (row) for each model for the given "newdata".
+#'      \item Each model's feature-level SHAP ratios (i.e., share of total SHAP) are computed.
+#'      \item The performance metrics of the models are used as weights.
+#'      \item Using the weights vector and shap ratio of features for each model,
+#'            the weighted mean SHAP ratios and their confidence intervals are computed.
+#'    }
+#'
+#' @param models h2o search grid, autoML grid, or a character vector of H2O model IDs.
+#' @param newdata An \code{h2o} frame (or \code{data.frame}) already uploaded to the
+#'                \code{h2o} server. This data will be used for computing SHAP
+#'                contributions for each model, alongside model's performance
+#'                weights.
+#' @param performance_metric Character specifying which performance metric to use
+#'                           as weights. The default is \code{"r2"}, which can
+#'                           be used for both regression and classification.
+#'                           For binary classification, other options include:
+#'                           \code{"aucpr"} (area under the precision-recall curve),
+#'                           \code{"auc"} (area under the ROC curve),
+#'                           and \code{"f2"} (F2 score).
+#' @param standardize_performance_metric Logical, indicating whether to standardize
+#'                                       the performance metric used as weights so
+#'                                       their sum equals the number of models. The
+#'                                       default is \code{FALSE}.
+#' @param performance_type Character. Specify which performance metric should be
+#'                        reported: \code{"train"} for training data, \code{"valid"}
+#'                        for validation, or \code{"xval"} for cross-validation (default).
+#' @param minimum_performance Numeric. Specify the minimum performance metric
+#'                            for a model to be included in calculating weighted
+#'                            mean SHAP ratio Models below this threshold receive
+#'                            zero weight. The default is \code{0}.
+#' @param method Character. Specify the method for selecting important features
+#'               based on their weighted mean SHAP ratios. The default is
+#'               \code{"mean"}, which selects features whose weighted mean shap ratio (WMSHAP)
+#'               exceeds the \code{cutoff}. The alternative is
+#'               \code{"lowerCI"}, which selects features whose lower bound of confidence
+#'               interval exceeds the \code{cutoff}.
 #' @param cutoff numeric, specifying the cutoff for the method used for selecting
-#'               the top features. the default is zero, which means that all
-#'               features with the "method" criteria above zero will be selected.
+#'               the top features.
 #' @param top_n_features integer. if specified, the top n features with the
 #'                       highest weighted SHAP values will be selected, overrullung
 #'                       the 'cutoff' and 'method' arguments. specifying top_n_feature
@@ -69,8 +62,22 @@
 #'                 criterion in order to compute WMSHAP and CI. If the intention
 #'                 is to compute global summary SHAP values (at feature level) for
 #'                 a single model, set n_models to 1. The default is 10.
+#' @param sample_size integer. number of rows in the \code{newdata} that should
+#'                    be used for SHAP assessment. By default, all rows are used,
+#'                    which is the recommended procedure for scientific analyses.
+#'                    However, SHAP analysis is time consuming and in the process
+#'                    of code development, lower values can be used for quicker
+#'                    shapley analyses.
 #' @param plot logical. if TRUE, the weighted mean and confidence intervals of
 #'             the SHAP values are plotted. The default is TRUE.
+#'
+# @param normalize_shap_per_model Logical. If TRUE, the SHAP contribution ratio for each
+#                        model is normalized to range between 0 to 1. The default is
+#                        FALSE, which encourages reporting Weighted Mean SHAP Ratios (not ranks)
+#                        comparison between models without scaling each model.
+#                        This option is only provided for rare use cases where
+#                        WMSHAP values are prefered to be scaled to 0 to 1, but
+#                        should otherwise be avoided.
 # @param normalize_to character. The default value is "upperCI", which sets the feature with
 #                     the maximum SHAP value to one, allowing the higher CI to
 #                     go beyond one. Setting this value is mainly for aesthetic
@@ -81,6 +88,7 @@
 #                     in the normalization of the SHAP values, the maximum confidence
 #                     interval of the weighted SHAP values should be equal to
 #                     "1", in order to limit the plot values to maximum of one.
+#'
 #' @importFrom utils setTxtProgressBar txtProgressBar globalVariables
 #' @importFrom stats weighted.mean
 #' @importFrom h2o h2o.stackedEnsemble h2o.getModel h2o.auc h2o.aucpr h2o.r2
@@ -90,7 +98,6 @@
 #' @importFrom curl curl
 #' @importFrom ggplot2 ggplot aes geom_col geom_errorbar coord_flip ggtitle xlab
 #'             ylab theme_classic theme scale_y_continuous margin expansion
-#' @author E. F. Haghish
 #' @return a list including the GGPLOT2 object, the data frame of SHAP values,
 #'         and performance metric of all models, as well as the model IDs.
 #' @examples
@@ -161,6 +168,7 @@
 #'
 #' }
 #' @export
+#' @author E. F. Haghish
 
 shapley <- function(models,
                     newdata,
@@ -170,10 +178,12 @@ shapley <- function(models,
                     standardize_performance_metric = FALSE,
                     performance_type = "xval",
                     minimum_performance = 0,
-                    method = c("lowerCI"),
-                    cutoff = 0.0,
+                    method = "mean",
+                    cutoff = 0.01,
                     top_n_features = NULL,
-                    n_models = 10
+                    n_models = 10,
+                    sample_size = nrow(newdata)
+                    #normalize_shap_per_model = FALSE
                     #normalize_to = "upperCI"
 ) {
 
@@ -196,6 +206,8 @@ shapley <- function(models,
   valid <- FALSE
   xval  <- FALSE
 
+  #if (normalize_shap_per_model) message("using 'normalize_shap_per_model' is discouraged! This is for rare use only")
+
   # Syntax check
   # ============================================================
   if (performance_metric != "r2" &
@@ -348,7 +360,6 @@ shapley <- function(models,
     lowerCI = NA,
     upperCI = NA)
 
-
   # CALCULATE THE TOTAL CONTRIBUTION PER MODEL (don't collapse to vector)
   TOTAL <- colSums(abs(results[, grep("^contribution", names(results)), FALSE]),
                    na.rm = TRUE)
@@ -366,15 +377,13 @@ shapley <- function(models,
 
   # Scale the ratio matrix and create a data frame
   # -------------------------------------------------------------
-  for (i in 1:ncol(ratioDF)) {
-    ratioDF[,i] <- normalize(abs(ratioDF[,i]), min = 0)
-  }
+  for (i in 1:ncol(ratioDF)) ratioDF[,i] <- abs(ratioDF[,i])
   ratioDF <- as.data.frame(ratioDF)
   names(ratioDF) <- paste0("ratio", 1:ncol(ratioDF))
-  feature <- unique(summaryShaps$feature)              #??? summaryShaps should be empty by now???
+  feature <- unique(summaryShaps$feature)
   ratioDF <- cbind(feature, ratioDF)
 
-  # Cmpute the weighted mean, sd, and ci for each feature
+  # Compute the weighted mean, sd, and ci for each feature
   # -------------------------------------------------------------
   for (j in UNQ) {
     # get all contribution columns for the j feature
@@ -438,7 +447,7 @@ shapley <- function(models,
   #                                         min = min,
   #                                         max = max)
   # compute relative shap values
-  summaryShaps$shapratio <- summaryShaps$mean / sum(summaryShaps$mean)
+  # summaryShaps$shapratio <- summaryShaps$mean / sum(summaryShaps$mean)
 
   # # compute lowerCI
   # summaryShaps$lowerCI <- summaryShaps$normalized_mean - summaryShaps$normalized_ci
@@ -454,13 +463,13 @@ shapley <- function(models,
     if (method == "mean") {
       selectedFeatures <- selectedFeatures[selectedFeatures$mean > cutoff, ]
     }
-    else if (method == "shapratio") {
-      selectedFeatures <- selectedFeatures[selectedFeatures$shapratio > cutoff, ]
-    }
+    # else if (method == "shapratio") {
+    #   selectedFeatures <- selectedFeatures[selectedFeatures$shapratio > cutoff, ]
+    # }
     else if (method == "lowerCI") {
       selectedFeatures <- selectedFeatures[selectedFeatures$lowerCI > cutoff, ]
     }
-    else stop("method must be one of 'mean', 'shapratio', or 'ci'")
+    else stop("method must be one of 'mean' or 'lowerCI'")
   }
 
 
